unit uProxy;

interface

uses
  DBClient, uSubject, Data.DBXJSONReflect;

type
  { Proxy }
  TCalculadorProxy = class(TInterfacedObject, ICalculador)
  private
    // Armazena uma referncia para o Real Subject (objeto real)
    CalculadorReal: ICalculador;

    // DataSet para armazenar os dados de cache
    CacheDados: TClientDataSet;
  public
    constructor Create;

    // Mtodo da Interface
    function CalcularDistancia(const Origem, Destino: string): string;
  end;

implementation

uses
  DB, Variants, SysUtils, Forms, uRealSubject;

{ TCalculadorProxy }

function TCalculadorProxy.CalcularDistancia(const Origem,
  Destino: string): string;
begin
  // Verifica se o valor da distncia est no DataSet de cache
  if CacheDados.Locate('Origem;Destino', VarArrayOf([Origem, Destino]), []) then
  begin
    // Se o valor estiver no DataSet, no  necessrio chamar o objeto real
    result := CacheDados.FieldByName('Distancia').AsString;
    Exit;
  end;

  // Cria a instncia objeto real (Real Subject) caso ela ainda no exista
  // Obs: a criao  sob demanda
  if not Assigned(CalculadorReal) then
    CalculadorReal := TCalculadorReal.Create;

  // Chama o objeto real para obter a distncia usando a API do Google Maps
  result := CalculadorReal.CalcularDistancia(Origem, Destino);

  // Adiciona os dados no DataSet de cache
  // para evitar uma nova requisio  API, aumentando o desempenho da aplicao
  CacheDados.AppendRecord([Origem, Destino, result]);

  // Salva o arquivo de cache em disco
  CacheDados.SaveToFile(ExtractFilePath(Application.ExeName) + 'Cache.xml');
end;

constructor TCalculadorProxy.Create;
var
  ArquivoCache: string;
begin
  // Cria o DataSet de cache (tabela temporria)
  CacheDados := TClientDataSet.Create(nil);

  // Se o arquivo de cache existir,  carregado
  ArquivoCache := ExtractFilePath(Application.ExeName) + 'Cache.xml';
  if FileExists(ArquivoCache) then
    CacheDados.LoadFromFile(ArquivoCache)
  else
  begin
    // Caso contrrio, a estrutura do DataSet  criada para ser usado pela primeira vez
    // ou a cada vez que o cache for excludo do diretrio da aplicao
    CacheDados.FieldDefs.Add('Origem', ftString, 50);
    CacheDados.FieldDefs.Add('Destino', ftString, 50);
    CacheDados.FieldDefs.Add('Distancia', ftString, 10);
    CacheDados.CreateDataSet;
  end;

  // Desliga o log de alteraes
  CacheDados.LogChanges := False;
end;

end.
